<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet; 
use Teknisa\MAN\Services\ParametroGerencial as ParametroGerencialService;

class ParametroGerencial {
    
    protected $utilService;
    private $nrorg;
    private $unidades;
    private $lojas;
    
    public function __construct (Utilities\Util $utilService, ParametroGerencialService $parametroGerencialService) {
        $this->utilService = $utilService;
        $this->parametroGerencialService = $parametroGerencialService;
    }
    
    public function importarArquivo (DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);
        $dados = $row['ARQUIVOXLS'][0]['b64File'];
        $tipo = $row['TPARQUIVO'];
        $this->nrorg = $row['NRORG'];
        $importacao = $this->getInformacoesArquivo($dados);
        $resultado = array();
        $dadosRecuperados = true;
        $parametroId = 1;
        
        date_default_timezone_set('America/Sao_Paulo');
        $resultado[] = "Log de Importação de Parâmetros P&L Gerencial";
        $resultado[] = "Arquivo: ".$row['ARQUIVOXLS'][0]['name'];
        
        switch ($tipo) {
        case 'G':
            $resultado[] = "Tipo do Arquivo: Geral";
            break;
        case 'D':
            $resultado[] = "Tipo do Arquivo: Despesa";
            break;
        case 'R':
            $resultado[] = "Tipo do Arquivo: Receita";
            break;
        }
        
        $resultado[] = "Data: ".date("d/m/Y H:i:s");
        $resultado[] = "Registros detectados: ".sizeof($importacao);
        $resultado[] = "";
        
        $dadosRecuperados = $this->recuperarInformacoesNecessarias($resultado, $tipo);
        
        if (!$this->checarTipoArquivo(array_keys($importacao[0]), $tipo)) {
            $dadosRecuperados = false;
            
            $resultado[] = "O arquivo não está dentro dos padrões.";
            $resultado[] = "Verique se o tipo do arquivo está correto e tente novamente.";
            $resultado[] = "A operação foi abortada.";
        }
        
        if ($dadosRecuperados) {
            foreach ($importacao as $dados) {
                try {
                    
                    $cdfilial = $dados['CDFILIAL'];
                    $cdloja = $dados['CDLOJA'];
                    
                    
                    $resultado[] = "Registro: ".str_pad($parametroId, 3, "0", STR_PAD_LEFT);
                    $resultado[] = "Unidade: ".$cdfilial." - Loja: ".$cdloja." - Período: ".$dados['DTPERIODO'];
                    $parametroId++;
                    $registroValido = $this->validarRegistro($resultado, $dados, $tipo);
      
                    if ($registroValido) {
                        switch ($tipo) {
                        case 'G':
                            $resp = $this->parametroGerencialService->deletarTodasReceitas($dados);
                            $resp = $this->parametroGerencialService->deletarTodasDespesas($dados);
                            $this->parametroGerencialService->deletarRegistrosGerais($dados);
                            $this->parametroGerencialService->inserirParametrosGerais($dados);
                            $resultado[] = "Parametrização importada com sucesso.";
                            break;
                        case 'D':
                            $resultado[] = "Código da despesa: ".str_pad($dados['CDTPCTPAGAR'], 6, "0", STR_PAD_LEFT);
                            $this->parametroGerencialService->deletarDespesas($dados);
                            $this->parametroGerencialService->inserirDespesas($dados);
                            $resultado[] = "Despesa importada com sucesso.";
                            break;
                        case 'R':
                            $resultado[] = "Código da receita: ".str_pad($dados['CDTPCTRECEB'], 6, "0", STR_PAD_LEFT);
                            $this->parametroGerencialService->deletarReceitas($dados);
                            $this->parametroGerencialService->inserirReceitas($dados);
                            $resultado[] = "Receita importada com sucesso.";
                            break;
                        }
                    }
                    
                    $resultado[] = str_pad('-', 100, "-", STR_PAD_LEFT);
                } catch(\Exception $e){
                    $resultado[] = "Ocorreu um erro ao inserir o registro.";
                    if (strpos($e->getMessage(), 'ORA-02292')) {
                        
                        $resultado[] = "Verifique se esse registro não possui nenhum registro associado a ele.";
                        $resultado[] = "Delete os registros com o mesmo período, unidade e loja e tente novamente.";
                        $resultado[] = "Código do erro: ORA-02292";
                        
                    } else if (strpos($e->getMessage(), 'ORA-02291')) {
                        
                        $resultado[] = "Verifique se já inseriu a parametrização geral antes de inserir esse registro.";
                        $resultado[] = "Tente inserir a parametrização Geral e tente novamente.";
                        $resultado[] = "Código do erro: ORA-02291";
                        
                    } else {
                        $resultado[] = "Erro desconhecido.";
                        if ($e->getCode() == 0) {
                            $resultado[] = $e->getMessage();
                        } else
                            $resultado[] = "Código do erro: ".$e->getCode();
                    }
                    $resultado[] = str_pad('-', 100, "-", STR_PAD_LEFT);
                    // $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
                }
            }
        }
      
        
        $response->addDataset(new DataSet('log', $resultado));
        
    }
    
    private function checarTipoArquivo ($keys, $tipo) {
        
        $resultado = true;
        
        $tipos = array(
            'G' => ['CDFILIAL', 'CDLOJA', 'DTPERIODO', 'VRSALARIOMENSAL', 'VRPERCFIXMARKET', 'VRPERCFIXROYALT',
                    'NRTOTALHORAS',	'VRALUGUEL', 'VRCONDOMINIO', 'VRSEMITOTREAL', 'VRBRUTVENDPROJ',	'VRMENSALPROJ',	
                    'VRQTHORAPROJ',	'VRCUSTOHRPROJ', 'VRPERCMARKPROJ', 'VRPERCROYPROJ',	'VRALUGUELPROJ', 'VRCONDOMPROJ',
                    'VRSEMITOTPROJ', 'VRQTTRANSPROJ', 'VRQTCLIPROJ', 'VRESTVALORIZADO', 'VRFRETECAR', 'VRFRETECARPROJ', 'VRARMAZENA', 'VRARMAZENAPROJ'],
            'D' => ['CDFILIAL', 'CDLOJA', 'DTPERIODO', 'CDTPCTPAGAR', 'VRCTPAGAR', 'IDUTILITIES', 'VRDESPESAPROJ', 'IDSUBTRAISEMIS'],
            'R' => ['CDFILIAL',	'CDLOJA', 'DTPERIODO', 'CDTPCTRECEB', 'VRCTRECEB', 'VRRECEITAPROJ']
        );
        
        $i = 0;
        
        foreach ($keys as $key) {
            if(in_array($key, $tipos[$tipo])) {
                $i++;
            }
            $inicioString = substr($key, 0,2);
        }
        
        if($i != sizeof($tipos[$tipo]))
            $resultado = false;
            
        return $resultado;
        
    }
    
    private function validarRegistro (&$log, $registro, $tipo) {
        
        $resultado = true;
        $cdfilial = $registro['CDFILIAL'];
        $cdloja = $registro['CDLOJA'];
        $periodo = $registro['DTPERIODO'];
        
        if (empty($cdfilial) || $cdfilial == null) {
            $log[] = "CDFILIAL está vazio.";
            $resultado = false;
        } else {
            if(!in_array($cdfilial, $this->unidades)) {
                $log[] = "Unidade não existe.";
                $resultado = false;
            }
        }
        
        if (empty($cdloja) || $cdloja == null) {
            $log[] = "CDLOJA está vazio.";
            $resultado = false;
        } else {
            if(!in_array($cdloja."_".$cdfilial, $this->lojas)) {
                $log[] = "Loja não existe.";
                $resultado = false;
            }
        }
        
        if (empty($periodo) || $periodo == null) {
            $log[] = "DTPERIODO está vazio.";
            $resultado = false;
        }
        
        
        switch ($tipo) {
            case 'G':
                break;
            case 'D':
                $cd = $registro['CDTPCTPAGAR'];
                if (empty($cd) || $cd == null || !isset($cd)) {
                    $log[] = "CDTPCTPAGAR está vazio.";
                    $resultado = false;
                } else {
                    $cd = str_pad($registro['CDTPCTPAGAR'], 6, "0", STR_PAD_LEFT);
                    if (!in_array($cd, $this->despesas)) {
                        $log[] = "Código da despesa não existe.";
                        $resultado = false;
                    } 
                }
                
                if (empty($registro['IDUTILITIES']) || $registro['IDUTILITIES'] == null || !isset($registro['IDUTILITIES'])) {
                    $log[] = "Valor do campo IDUTILITIES não existe.";
                    $resultado = false;
                }
                
                if (empty($registro['IDSUBTRAISEMIS']) || $registro['IDSUBTRAISEMIS'] == null || !isset($registro['IDSUBTRAISEMIS'])) {
                    $log[] = "Valor do campo IDSUBTRAISEMIS não existe.";
                    $resultado = false;
                }
                break;
            case 'R':
                $cd = $registro['CDTPCTRECEB'];
                if (empty($cd) || $cd == null || !isset($cd)) {
                    $log[] = "CDTPCTRECEB está vazio.";
                    $resultado = false;
                } else {
                    $cd = str_pad($registro['CDTPCTRECEB'], 6, "0", STR_PAD_LEFT);
                    if (!in_array($cd, $this->receitas)) {
                        $log[] = "Código da receita não existe.";
                        $resultado = false;
                    }
                }
                break;
        }
        
        $chaves = array_keys($registro);
        
        foreach ($chaves as $chave) {
            $tipoCampo = substr($chave, 0, 2); 
            
            if($tipoCampo == 'VR') {
                if($registro[$chave] != null and !empty($registro[$chave])) {
                    if (!is_numeric($registro[$chave])) {
                        $log[] = "Não é um número. Campo: ".$chave.". Valor: ".$registro[$chave];
                        $resultado = false;
                    } else {
                        if ($registro[$chave] < 0) {
                            $log[] = "Alerta! Valor negativo encontrado em: ".$chave.". Valor: ".$registro[$chave];
                            $resultado = false;
                        }
                    }
                }
            } else if ($tipoCampo == 'ID') {
                if ($registro[$chave] != null and !empty($registro[$chave]) ) {
                    if (strlen($registro[$chave]) > 1 || ($registro[$chave] != 'S' && $registro[$chave] != 'N')) {
                        $log[] = "Valor incorreto encontrado em: ".$chave.". Utilize S ou N ao invés de: '".$registro[$chave]."'";
                        $resultado = false;
                    }
                }
            }
        }
        
        
        return $resultado;
    }
    
    private function recuperarInformacoesNecessarias(&$resultado, $tipo) {
        
        $dadosRecuperados = true;
        
        try {
            $this->unidades = $this->parametroGerencialService->recuperarUnidades($this->nrorg);
        } catch(\Exception $e){
            $resultado[] = "Não foi possível recuperar as Unidades dessa organização.";
            $resultado[] = "A operação foi abortada.";
            $resultado[] = "Código do erro: ".$e->getCode();
            $dadosRecuperados = false;
        }
        
        try {
            $this->lojas = $this->parametroGerencialService->recuperarLojas($this->nrorg);
        } catch(\Exception $e){
            $resultado[] = "Não foi possível recuperar as Lojas dessa organização.";
            $resultado[] = "A operação foi abortada.";
            $resultado[] = "Código do erro: ".$e->getCode();
            $dadosRecuperados = false;
        }
        
        if ($tipo == 'D') {
            try {
                $this->despesas = $this->parametroGerencialService->recuperarDespesas($this->nrorg);
            } catch(\Exception $e){
                $resultado[] = "Não foi possível recuperar as Despesas dessa organização.";
                $resultado[] = "A operação foi abortada.";
                $resultado[] = "Código do erro: ".$e->getCode();
                $dadosRecuperados = false;
            }
        } else if ($tipo == 'R') {
             try {
                $this->receitas = $this->parametroGerencialService->recuperarReceitas($this->nrorg);
            } catch(\Exception $e){
                $resultado[] = "Não foi possível recuperar as Receitas dessa organização.";
                $resultado[] = "A operação foi abortada.";
                $resultado[] = "Código do erro: ".$e->getCode();
                $dadosRecuperados = false;
            }
        }
        
        return $dadosRecuperados;
    }
    
    private function getInformacoesArquivo ($string) {
        $b64 = explode(",", $string)[1];
        $i = 1;
        
        $fileStr = base64_decode($b64);
        $fileStr = str_replace('"', '', $fileStr);
        $fileArray = explode("\n", $fileStr);
        $informacoes = array();
        
        // $fileStr = str_replace('#!', '', $fileStr);
        $keys = explode(",", $fileArray[0]);
    
        for ($i = 1; $i < sizeof($fileArray); $i++) {
            if(strlen(($fileArray[$i])) > 0) {
                $string = $fileArray[$i];
                $obj = array();
                $valores = explode(",", $string);
                
                for ($j = 0; $j < sizeof($keys); $j++) {
                    $obj[trim($keys[$j])] = trim($valores[$j]);
                } 
                $obj['NRORG'] = $this->nrorg;
                $informacoes[] = $obj;
            }
        }  

        return $informacoes;
    }
	
}