<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\Util\Functions;

class RelVendasProdutoPeriodoporModo{
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getIntervalo($params) {
        $resultQuery = $this->entityManager->getConnection()->fetchAll(self::GET_INTERVALO, $params);
        
        for($i = 0; $i < count($resultQuery); $i++) {
            $produtos[] = $resultQuery[$i]['CDARVPROD'];
        }
        
        $produtos = implode('_', $produtos);
        
        return $produtos;
    }
    
    public function buildParamsForRel($conditions){
        $conditions = $this->addParams($conditions,'CDFILIAL');
        $conditions = $this->addParams($conditions,'CDLOJA');
        $conditions = $this->addParams($conditions,'DAYSOFWEEK');
        $conditions = $this->buildParamsDataAbertur($conditions);
        $conditions = $this->buildParamsModalidade($conditions);
        $conditions = $this->builParamsProduto($conditions);
        return $conditions;
        
    }
    
    private function buildParamsModalidade($conditions){
        if(!Functions::arrayKeyExists('MODALIDADE', $conditions)) {
            $conditions['TMODALIDADE'] = 'T';
            $conditions['MODALIDADE'] = null;
        } else {
            $conditions['MODALIDADE'] = implode('_', $conditions['MODALIDADE']);
            $conditions['TMODALIDADE'] = null;
        }
        
        $conditions['FLAGBALTAA'] = 'N';
        $conditions['FLAGBALTAA'] = (strpos($conditions['MODALIDADE'], 'BAL') !== false ? (strpos($conditions['MODALIDADE'], 'TAA')  !== false ? 'N' : 'S') : 'N');
        return $conditions;
    }
    
    private function buildParamsDataAbertur($conditions){
        if(Functions::arrayKeyExists('DTENTRVENDA', $conditions)) {
            $conditions['DTINI'] = $conditions['DTENTRVENDA'][0];
            $conditions['DTFIM'] = $conditions['DTENTRVENDA'][1];
        }
        unset($conditions['DTENTRVENDA']);
        return $conditions;
    }
    
    private function builParamsProduto($conditions){
        if(!Functions::arrayKeyExists('CDARVPRODI', $conditions) && 
            !Functions::arrayKeyExists('CDARVPRODF', $conditions) &&
            !Functions::arrayKeyExists('CDARVPROD', $conditions)) {
            
            $conditions['CDPRODINI'] = $conditions['CDPRODFIM'] = null;    
            $conditions['TCDPRODUTO'] = 'T';
            $conditions['CDPRODS'] = null;
        } else {
            if(Functions::arrayKeyExists('CDARVPRODI', $conditions) && 
                !Functions::arrayKeyExists('CDARVPRODF', $conditions)) {
                $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
            }
            
            if(!Functions::arrayKeyExists('CDARVPRODI', $conditions) && 
                !Functions::arrayKeyExists('CDARVPRODF', $conditions)) {
                
                
                $conditions['CDPRODS'] = $this->getProdutos($conditions['CDARVPROD']);
                unset($conditions['CDARVPROD']);
            } else {
                $params = array(
                    ':CDPRODINI' => str_replace("%", "", $conditions['CDARVPRODI']),
                    ':CDPRODFIM' => str_replace("%", "", $conditions['CDARVPRODF'])
                );
                
                $conditions['CDPRODS'] = $this->getIntervalo($params);
            }
            
            $conditions['TCDPRODUTO'] = null;
            unset($conditions['CDARVPRODI']);
            unset($conditions['CDARVPRODF']);
            unset($conditions['NMPRODUTO']);
        }
        return $conditions;
    }
    
    private function getProdutos($params){
        $conditions = '';
        if(!empty($params)){
          $conditions = implode('_', $params);  
        }
        return $conditions;
    }
    
    private function addParams($conditions,$param){
        
        if(!Functions::arrayKeyExists($param, $conditions)) {
            $conditions[$param] = '';
            $conditions['T'.$param] = 'T';
        }else{
            $conditions[$param] = implode('_', $conditions[$param]);
            $conditions['T'.$param] = '';
        }
        return $conditions;
    }
    
    public function findVendasProdMod($params){
        $tipo = Functions::arrayKeyExists('TIPORELATORIO', $params) ? $params['TIPORELATORIO'] : 'A';
        return $this->entityManager->getConnection()->fetchAll($this->getQuery($tipo), $params);
    }
    
    private function getQuery($tipo){
        
        if($tipo === 'A'){
            return self::GET_VENDAS_PROD_MOD_ANALITICO;
        }
         else if($tipo === 'S'){
            return self::GET_VENDAS_PROD_MOD_SINTATICO;
        }
        else{
            return self::GET_VENDAS_PROD_MODALIDADE_ANALITICO;
        }
    }
    
    public function getMAXCDNVPRODUTO (){
        $result = $this->entityManager->getConnection()->fetchAll(self::GET_MAXCDNVPRODUTO);
        return $result[0]['CDNVPRODUTO'];
    }
    
    const GET_MAXCDNVPRODUTO = "
    SELECT 
      MAX(CDNVPRODUTO) AS CDNVPRODUTO 
        FROM PRODUTO 
    ";
    
    const GET_INTERVALO = "
        SELECT CDARVPROD FROM PRODUTO WHERE CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIM
    ";
    
    const GET_VENDAS_PROD_MOD_SINTATICO = "
        SELECT :CDFILIAL AS LABELFILIAL, P.NMPRODUTO, P.CDARVPROD, P.CDARVPROD AS LABELARV, 
        I.VRUNITVEND, SUM(I.QTPRODVEND) AS QTPRODVEND, :TIMEINI||' - '||:TIMEFIM AS TIMEINI,
        SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS VRTOTAL,
        TRUNC(((SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) * 100)/
                (
                  SELECT SUM(TRUNC(I2.QTPRODVEND * (I2.VRUNITVEND + NVL(I2.VRUNITVENDCL, 0)), 2) - NVL(I2.VRDESITVEND, 0) + NVL(I2.VRACRITVEND, 0))
                    FROM VENDA V2
                      JOIN ITEMVENDA I2 
                        ON V2.NRSEQVENDA  = I2.NRSEQVENDA 
                          AND V2.CDFILIAL = I2.CDFILIAL 
                          AND V2.CDCAIXA  = I2.CDCAIXA 
                          AND V2.NRORG    = I2.NRORG
                      JOIN PRODUTO P2 
                        ON P2.CDPRODUTO = I2.CDPRODUTO 
                          AND I2.NRORG  = P2.NRORG
                      JOIN LOJA L2
                        ON V2.CDLOJA = L2.CDLOJA 
                          AND V2.CDFILIAL = L2.CDFILIAL 
                          AND V2.NRORG    = L2.NRORG
                      LEFT JOIN ORIGEMVENDAFOS O2
                        ON V2.IDORIGEMVENDA = O2.IDORIGEMVENDA
                          AND V2.NRORG = O2.NRORG
                    WHERE V2.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
                      AND TO_CHAR(I2.DTHRPEDIDO, 'HH24') BETWEEN SUBSTR(:TIMEINI, 0, 2) AND SUBSTR(:TIMEFIM, 0, 2)
                      AND TO_CHAR(I2.DTHRPEDIDO, 'HH24:MI') BETWEEN :TIMEINI AND :TIMEFIM
                      AND (V2.NRORG = :NRORG)
                      AND (INSTR(:CDFILIAL, V2.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T'))
                      AND (INSTR(:DAYSOFWEEK, TO_CHAR(V2.DTENTRVENDA, 'D')) <> 0 OR (:TDAYSOFWEEK = 'T'))
                      AND ((INSTR(:CDLOJA, V2.CDFILIAL || V2.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                      AND V2.IDSITUVENDA = 'O'
                      AND I2.CDPRODUTO <> NVL(L2.CDPRODTAXASERV, 0)
                      AND I2.CDPRODUTO <> NVL(L2.CDPRODTAXAENTR, 0)
                      AND I2.VRUNITVEND > 0
                      AND ((:TCDPRODUTO = 'T') OR (INSTR(:CDPRODS, P2.CDARVPROD) <> 0 AND CDNVPRODUTO = :CDNVPRODUTO))
                      AND ((INSTR(:MODALIDADE, O2.IDORIGEMVENDA) <> 0) OR (:TMODALIDADE = 'T'))
                      AND O2.IDORIGEMVENDA <> (CASE :FLAGBALTAA WHEN 'S' THEN 'BAL_AUT' ELSE 'ZZZZZZ' END)
                )), 2) AS PORCENTAGEM
          FROM VENDA V
            JOIN FILIAL F 
              ON V.CDFILIAL = F.CDFILIAL
                AND V.NRORG = F.NRORG
            JOIN LOJA L  
              ON V.CDLOJA = L.CDLOJA 
                AND V.CDFILIAL = L.CDFILIAL 
                AND V.NRORG    = L.NRORG
            JOIN ITEMVENDA I 
              ON V.NRSEQVENDA  = I.NRSEQVENDA 
                AND V.CDFILIAL = I.CDFILIAL 
                AND V.CDCAIXA  = I.CDCAIXA 
                AND V.NRORG    = I.NRORG
            JOIN PRODUTO P 
              ON P.CDPRODUTO = I.CDPRODUTO 
                AND P.NRORG  = I.NRORG
            LEFT JOIN ORIGEMVENDAFOS O
              ON V.IDORIGEMVENDA = O.IDORIGEMVENDA
                AND V.NRORG = O.NRORG
          WHERE V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
                 AND TO_CHAR(I.DTHRPEDIDO, 'HH24') BETWEEN SUBSTR(:TIMEINI, 0, 2) AND SUBSTR(:TIMEFIM, 0, 2)
                 AND TO_CHAR(I.DTHRPEDIDO, 'HH24:MI') BETWEEN :TIMEINI AND :TIMEFIM
                 AND (INSTR(:DAYSOFWEEK, TO_CHAR(V.DTENTRVENDA, 'D')) <> 0 OR (:TDAYSOFWEEK = 'T'))
                 AND (INSTR(:CDFILIAL, V.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T'))
                 AND ((INSTR(:CDLOJA, V.CDFILIAL || V.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                 AND ((:TCDPRODUTO = 'T') OR (INSTR(:CDPRODS, P.CDARVPROD) <> 0 AND CDNVPRODUTO = :CDNVPRODUTO))
                 AND ((INSTR(:MODALIDADE, O.IDORIGEMVENDA) <> 0) OR (:TMODALIDADE = 'T'))
                 AND O.IDORIGEMVENDA <> (CASE :FLAGBALTAA WHEN 'S' THEN 'BAL_AUT' ELSE 'ZZZZZZ' END)
                 AND V.NRORG = :NRORG
                 AND V.IDSITUVENDA = 'O'
                 AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 0)
                 AND I.CDPRODUTO <> NVL(L.CDPRODTAXAENTR, 0)
                 AND I.VRUNITVEND > 0
          GROUP BY P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND
        ORDER BY QTPRODVEND DESC, CDARVPROD
    ";
    
    const GET_VENDAS_PROD_MODALIDADE_ANALITICO  = "
         SELECT 'TODAS' AS LABELFILIAL, P.NMPRODUTO,  P.CDARVPROD, P.CDARVPROD AS LABELARV,
        :TIMEINI||' - '||:TIMEFIM AS TIMEINI, NVL(O.DSORIGEMVENDA, 'Modalidade Desconhecida') AS NMMODALIDADE,
        I.VRUNITVEND, SUM(I.QTPRODVEND) AS QTPRODVEND,
        SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS VRTOTAL,
        TRUNC(((SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) * 100) /
              ( 
                SELECT SUM(TRUNC(I2.QTPRODVEND * (I2.VRUNITVEND + NVL(I2.VRUNITVENDCL, 0)), 2) - NVL(I2.VRDESITVEND, 0) + NVL(I2.VRACRITVEND, 0))
                  FROM VENDA V2
                    JOIN ITEMVENDA I2
                      ON V2.NRSEQVENDA  = I2.NRSEQVENDA
                        AND V2.CDFILIAL = I2.CDFILIAL
                        AND V2.CDCAIXA  = I2.CDCAIXA
                        AND V2.NRORG    = I2.NRORG
                    JOIN PRODUTO P2
                      ON I2.CDPRODUTO = P2.CDPRODUTO
                        AND I2.NRORG  = P2.NRORG        
                    JOIN LOJA L2
                      ON V2.CDLOJA = L2.CDLOJA
                        AND V2.CDFILIAL = L2.CDFILIAL
                        AND V2.NRORG    = L2.NRORG
                    LEFT JOIN ORIGEMVENDAFOS O2
                        ON V2.IDORIGEMVENDA = O2.IDORIGEMVENDA
                          AND V2.NRORG = O2.NRORG
                  WHERE V2.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
                    AND TO_CHAR(I2.DTHRPEDIDO, 'HH24') BETWEEN SUBSTR(:TIMEINI, 0, 2) AND SUBSTR(:TIMEFIM, 0, 2)
                    AND TO_CHAR(I2.DTHRPEDIDO, 'HH24:MI') BETWEEN :TIMEINI AND :TIMEFIM
                    AND (INSTR(:DAYSOFWEEK, TO_CHAR(V2.DTENTRVENDA, 'D')) <> 0 OR (:TDAYSOFWEEK = 'T'))
                    AND V2.NRORG = :NRORG
                    AND (INSTR(:CDFILIAL, V2.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T'))
                    AND ((INSTR(:CDLOJA, V2.CDFILIAL || V2.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                    AND V2.IDSITUVENDA = 'O'
                    AND I2.CDPRODUTO <> NVL(L2.CDPRODTAXASERV, 0)
                    AND I2.CDPRODUTO <> NVL(L2.CDPRODTAXAENTR, 0)
                    AND I2.VRUNITVEND > 0
                    AND ((:TCDPRODUTO = 'T') OR (INSTR(:CDPRODS, P2.CDARVPROD) <> 0 AND CDNVPRODUTO = :CDNVPRODUTO))
                    AND ((INSTR(:MODALIDADE, O2.IDORIGEMVENDA) <> 0) OR (:TMODALIDADE = 'T'))
                    AND O2.IDORIGEMVENDA <> (CASE :FLAGBALTAA WHEN 'S' THEN 'BAL_AUT' ELSE 'ZZZZZZ' END)
              )), 2) AS PORCENTAGEM 
          FROM VENDA V
            JOIN FILIAL F
              ON V.CDFILIAL = F.CDFILIAL
                AND V.NRORG = F.NRORG
            JOIN LOJA L
              ON V.CDLOJA = L.CDLOJA
                AND V.CDFILIAL = L.CDFILIAL
                AND V.NRORG    = L.NRORG
            JOIN ITEMVENDA I
              ON V.NRSEQVENDA  = I.NRSEQVENDA
                AND V.CDFILIAL = I.CDFILIAL
                AND V.CDCAIXA  = I.CDCAIXA
                AND V.NRORG    = I.NRORG 
            JOIN PRODUTO P
              ON I.CDPRODUTO = P.CDPRODUTO
                AND I.NRORG  = P.NRORG
            LEFT JOIN ORIGEMVENDAFOS O
              ON V.IDORIGEMVENDA = O.IDORIGEMVENDA
                AND V.NRORG = O.NRORG
          WHERE V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
            AND TO_CHAR(I.DTHRPEDIDO, 'HH24') BETWEEN SUBSTR(:TIMEINI, 0, 2) AND SUBSTR(:TIMEFIM, 0, 2)
            AND TO_CHAR(I.DTHRPEDIDO, 'HH24:MI') BETWEEN :TIMEINI AND :TIMEFIM
            AND (INSTR(:DAYSOFWEEK, TO_CHAR(V.DTENTRVENDA, 'D')) <> 0 OR (:TDAYSOFWEEK = 'T'))
            AND V.NRORG = :NRORG  
            AND V.IDSITUVENDA = 'O'
            AND (INSTR(:CDFILIAL, V.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T'))
            AND ((INSTR(:CDLOJA , V.CDFILIAL ||V.CDLOJA) <> 0 ) OR ( :TCDLOJA = 'T' ))
            AND ((:TCDPRODUTO = 'T')  OR (INSTR(:CDPRODS, P.CDARVPROD) <> 0 AND CDNVPRODUTO = :CDNVPRODUTO))
            AND ((INSTR(:MODALIDADE, O.IDORIGEMVENDA) <> 0) OR (:TMODALIDADE = 'T'))
            AND O.IDORIGEMVENDA <> (CASE :FLAGBALTAA WHEN 'S' THEN 'BAL_AUT' ELSE 'ZZZZZZ' END)
            AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 0)
            AND I.CDPRODUTO <> NVL(L.CDPRODTAXAENTR, 0)
            AND I.VRUNITVEND > 0
          GROUP BY P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, O.DSORIGEMVENDA
        ORDER BY QTPRODVEND DESC, CDARVPROD
    ";
    
    const GET_VENDAS_PROD_MOD_ANALITICO = "
     SELECT F.CDFILIAL || ' - ' || F.NMFILIAL ||' - Loja: ' ||L.CDLOJA || ' - ' || L.NMLOJA AS LABELFILIAL,
        P.NMPRODUTO,  P.CDARVPROD, P.CDARVPROD AS LABELARV,
        :TIMEINI||' - '||:TIMEFIM AS TIMEINI,
        NVL(O.DSORIGEMVENDA, 'Modalidade Desconhecida') AS NMMODALIDADE,
        V.CDFILIAL, I.VRUNITVEND, SUM(I.QTPRODVEND) AS QTPRODVEND,
        SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS VRTOTAL,
        TRUNC(((SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) * 100) /
            (SELECT SUM(TRUNC(I2.QTPRODVEND * (I2.VRUNITVEND + NVL(I2.VRUNITVENDCL, 0)), 2) - NVL(I2.VRDESITVEND, 0)  + NVL(I2.VRACRITVEND, 0))
              FROM VENDA V2
                JOIN ITEMVENDA I2
                  ON V2.NRSEQVENDA  = I2.NRSEQVENDA
                    AND V2.CDFILIAL = I2.CDFILIAL
                    AND V2.CDCAIXA  = I2.CDCAIXA
                    AND V2.NRORG    = I2.NRORG
                JOIN PRODUTO P2
                  ON I2.CDPRODUTO = P2.CDPRODUTO
                    AND I2.NRORG  = P2.NRORG        
                JOIN LOJA L2
                  ON V2.CDLOJA      = L2.CDLOJA
                    AND V2.CDFILIAL = L2.CDFILIAL
                    AND V2.NRORG    = L2.NRORG
                LEFT JOIN ORIGEMVENDAFOS O2
                  ON V2.IDORIGEMVENDA = O2.IDORIGEMVENDA
                    AND V2.NRORG = O2.NRORG
              WHERE V2.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
                AND TO_CHAR(I2.DTHRPEDIDO, 'HH24') BETWEEN SUBSTR(:TIMEINI, 0, 2) AND SUBSTR(:TIMEFIM, 0, 2)
                AND TO_CHAR(I2.DTHRPEDIDO, 'HH24:MI') BETWEEN :TIMEINI AND :TIMEFIM
                AND (INSTR(:DAYSOFWEEK, TO_CHAR(V2.DTENTRVENDA, 'D')) <> 0 OR (:TDAYSOFWEEK = 'T'))
                AND V2.NRORG = :NRORG
                AND (INSTR(:CDFILIAL, V2.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T'))
                AND ((INSTR(:CDLOJA , V2.CDFILIAL || V2.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                AND V2.IDSITUVENDA = 'O'
                AND I2.CDPRODUTO <> NVL(L2.CDPRODTAXASERV, 0)
                AND I2.CDPRODUTO <> NVL(L2.CDPRODTAXAENTR, 0)
                AND I2.VRUNITVEND > 0
                AND ((:TCDPRODUTO = 'T')  OR (INSTR(:CDPRODS, P2.CDARVPROD) <> 0 AND CDNVPRODUTO = :CDNVPRODUTO))
                AND ((INSTR(:MODALIDADE, O2.IDORIGEMVENDA) > 0) OR (:TMODALIDADE = 'T'))
                AND O2.IDORIGEMVENDA <> (CASE :FLAGBALTAA WHEN 'S' THEN 'BAL_AUT' ELSE 'ZZZZZZ' END)
            )), 2) AS PORCENTAGEM 
          FROM VENDA V
            JOIN FILIAL F
              ON V.CDFILIAL = F.CDFILIAL
                AND V.NRORG = F.NRORG
            JOIN LOJA L
              ON V.CDLOJA = L.CDLOJA
                AND V.CDFILIAL = L.CDFILIAL
                AND V.NRORG    = L.NRORG
            JOIN ITEMVENDA I
              ON V.NRSEQVENDA  = I.NRSEQVENDA
                AND V.CDFILIAL = I.CDFILIAL
                AND V.CDCAIXA  = I.CDCAIXA
                AND V.NRORG    = I.NRORG 
            JOIN PRODUTO P
              ON I.CDPRODUTO = P.CDPRODUTO
                AND I.NRORG  = P.NRORG
            LEFT JOIN ORIGEMVENDAFOS O
              ON V.IDORIGEMVENDA = O.IDORIGEMVENDA
                AND V.NRORG = O.NRORG
          WHERE V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
            AND TO_CHAR(I.DTHRPEDIDO, 'HH24') BETWEEN SUBSTR(:TIMEINI, 0, 2) AND SUBSTR(:TIMEFIM, 0, 2)
            AND TO_CHAR(I.DTHRPEDIDO, 'HH24:MI') BETWEEN :TIMEINI AND :TIMEFIM
            AND (INSTR(:DAYSOFWEEK, TO_CHAR(V.DTENTRVENDA,'D')) <> 0 OR (:TDAYSOFWEEK = 'T'))
            AND V.NRORG = :NRORG  
            AND V.IDSITUVENDA = 'O'
            AND (INSTR(:CDFILIAL, V.CDFILIAL) <> 0 OR (:TCDFILIAL = 'T'))
            AND ((INSTR(:CDLOJA, V.CDFILIAL || V.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
            AND ((:TCDPRODUTO = 'T') OR (INSTR(:CDPRODS, P.CDARVPROD) <> 0 AND CDNVPRODUTO = :CDNVPRODUTO))
            AND ((INSTR(:MODALIDADE, O.IDORIGEMVENDA) > 0) OR (:TMODALIDADE = 'T'))
            AND O.IDORIGEMVENDA <> (CASE :FLAGBALTAA WHEN 'S' THEN 'BAL_AUT' ELSE 'ZZZZZZ' END)
            AND I.CDPRODUTO <> NVL(L.CDPRODTAXASERV, 0)
            AND I.CDPRODUTO <> NVL(L.CDPRODTAXAENTR, 0)
            AND I.VRUNITVEND > 0
          GROUP BY F.CDFILIAL, F.NMFILIAL, L.NMLOJA, L.CDLOJA,  V.CDFILIAL, P.CDARVPROD, P.NMPRODUTO, I.VRUNITVEND, O.DSORIGEMVENDA
        ORDER BY CDFILIAL, QTPRODVEND DESC, CDARVPROD
    ";

}